% fit_multiple.m
%
% Script that calls fit_common.m for multiple trials of the same
% material sample to collect basic measurement statistics.  
%
% Inputs:
%       air_files       Cell array with filenames of air-only
%                       data files to use.
%       mat_files       Cell array with filenames of material
%                       measurement data.
%       The other parameters are the same as fit_common.

if ~exist('fit_multiple_keep_old', 'var'),
  fit_multiple_keep_old = 0;
end

if ~fit_multiple_keep_old,
eps_r_all = [];

for samp_num=1:length(air_files),
  fname_air = air_files{samp_num};
  fname_mat = mat_files{samp_num};
  
  fit_common;
  
  eps_r_all = [eps_r_all; eps_r_fit];
  
end
f_out = f_fit;
end


% Compute the mean eps_r and variance from the mean
N_cases = size(eps_r_all,1);
if (N_cases > 1),
  eps_r_mean = mean(eps_r_all);
else
  eps_r_mean = eps_r_all;
end
eps_r_real_mean = real(eps_r_mean);
eps_r_imag_mean = -imag(eps_r_mean);
eps_r_real_err = sqrt(mean(mean(abs(real(eps_r_all) - ones(N_cases,1)*eps_r_real_mean).^2)));

eps_r_imag_mean = mean(-imag(eps_r_all));
eps_r_imag_err = sqrt(mean(mean((-imag(eps_r_all) - ones(N_cases,1)*eps_r_imag_mean).^2)));

fprintf('RMS deviation:\n');
fprintf('Real(eps_r): %.4f (3 sigma = %.4f)\n', eps_r_real_err, 3*eps_r_real_err);
fprintf('Imag(eps_r): %.4f (3 sigma = %.4f)\n', eps_r_imag_err, 3*eps_r_imag_err);

% Do a best line fit of the material
p_r = polyfit(f_out/1e9, real(eps_r_mean), 1);
eps_r_line_fit_re = polyval(p_r, f_out/1e9);
p_i = polyfit(f_out/1e9, -imag(eps_r_mean), 1);
eps_r_line_fit_im = polyval(p_i, f_out/1e9);
eps_r_line_fit = eps_r_line_fit_re - j*eps_r_line_fit_im;

fprintf('Line Fit:\n');
fprintf('%3g GHz: %.3f - j%.3f\n', f_out(1)/1e9, real(eps_r_line_fit(1)), -imag(eps_r_line_fit(1)));
fprintf('%3g GHz: %.3f - j%.3f\n', f_out(end)/1e9, real(eps_r_line_fit(end)), -imag(eps_r_line_fit(end)));

%% Plot the results (if desired)

if (fit_multiple_do_plot),

figure(10);
[axl h1l h2l] = plotyy(f_out/1e9, real(eps_r_line_fit), f_out/1e9, -imag(eps_r_line_fit));
hold on;
[ax h1 h2] = plotyy(f_out/1e9, real(eps_r_all), f_out/1e9, -imag(eps_r_all));

[h3] = plot(res_est(:,2)/1e9, res_est(:,3), 'r*');
hold off;

set(h1, 'color', 'b');
set(h2, 'color', 'r');

set(h1l, 'color', 'k');
set(h1l, 'linewidth', 1.5);
set(h2l, 'color', 'k');
set(h2l, 'linewidth', 1.5);

set(ax(1),'YColor','k');
set(ax(1),'YTick', fit_multiple_y_tics);
set(ax(1),'YLim', [fit_multiple_y_tics(1) fit_multiple_y_tics(end)]);

set(axl(1),'YColor','k');
set(axl(1),'YTick', fit_multiple_y_tics);
set(axl(1),'YLim', [fit_multiple_y_tics(1) fit_multiple_y_tics(end)]);

set(ax(2),'YColor','k');
set(ax(2),'YTick', fit_multiple_y2_tics);
set(ax(2),'YLim', [fit_multiple_y2_tics(1) fit_multiple_y2_tics(end)]);

set(axl(2),'YColor','k');
set(axl(2),'YTick', fit_multiple_y2_tics);
set(axl(2),'YLim', [fit_multiple_y2_tics(1) fit_multiple_y2_tics(end)]);

if exist('fit_multiple_xlim', 'var'),
  set(ax(1), 'XLim', fit_multiple_xlim);
  set(ax(2), 'XLim', fit_multiple_xlim);
end

if exist('fit_multiple_title', 'var'),
  title(fit_multiple_title);
end
xlabel('Frequency (GHz)');
ylabel(ax(1), '\epsilon_r''');
ylabel(ax(2), '\epsilon_r''''');

set (h1, 'linestyle', '-');
set (h2, 'linestyle', '--');

set_fontsize(gcf, 18);

grid;
h = [h1(1) h2(1) h3 h1l];
legend(h, '\epsilon_r''', '\epsilon_r''''', '\epsilon_r'' (resonator)','Linear Fit', 'location', fit_multiple_legend_pos);

saveas(gcf, 'plot.eps', 'epsc');

end







