function [f_fit, eps_r_fit, d_fit] = fit_eps_data(f, S_air, S_mat, d, ops);

% [f_fit, eps_r_fit, d] = fit_eps_data(f, S_air, S_mat, d, ops);
%
% Uses measured data to estimate complex permittivity by fitting
% to a transmission line type model.
%
% Inputs:
%       f               VNA frequencies used
%       S_air           S21 for air measurement S(freq, snapshot)
%       S_mat           S21 for material measurement S(freq, snapshot)
%       d               Thickness of the sample (m)
%       ops             Options
%
% Options:
%       D               Total distance from antennas to mat. surface
%       model           Fitting model to use 'lin', 'cole'
%       N_iter          Number of iterations (for the 'lin' model)
%       ?_range         Structure containing parameter ranges to search.
%                       Each range is [start stop N_pts is_log] to
%                       indicate start, stop, # of points, and if
%                       a log scale should be used.
%       lin_range       Ranges: eps_real, eps_imag, band
%       cole_range      Ranges: eps_inf, eps_s, sig_s, alpha_s, tau_s
%       filt_type       Type of time-domain window to use:
%                       'kaiser'=single window for both air and material
%                       'kaiser2'=separate window for air and material
%       delta_w1        Time window left size
%       delta_w2        Time window right size
%       delta_tr        Time window rolloff
%       dev_pass_dB     Passband ripple in dB
%       dev_stop_dB     Stopband attenuation in dB
%
%       eps_r_exact     Exact eps_r.  If provided, will plot 
%                       model for the exact case.
%
%       spread_type     Take into account wave spreading (or not)
%                       0=No (1D), 1=3D spreading, 2=2D spreading
%       do_plot         Flag, if set generates plots
%                       1=mag, angle, 2=real, imag
%       ext_data        Frequency extend data to reduce time-gate artifacts
%                       [N_left N_right N_order]

c0 = 2.998e8;

%% Set default options

% Fitting model and parameters
model = 'lin';

% Number of iterations for the 'lin' model
N_iter = 5;

% Linear model fitting ranges
lin_range.eps_real = [1 10 1001 0];
lin_range.eps_imag = [0 10 1001 0];

% Cole model ranges to use
cole_range.eps_inf = [1 10 51 0];
cole_range.eps_s   = [1 10 51 0];
cole_range.sig_s   = [1e-8 1e-2 9 1];
cole_range.alpha_s = [0 1 11 0];
cole_range.tau_s   = [1e-14 1e-6 9 1];

% Time-gating parameters
filt_type = 'kaiser2';  % Use separate kaiser windows for air and material
% Size of the time-gating window
delta_w1 = 2e-2/c0;  % Time window width from peak to the left
delta_w2 = 10e-2/c0;  % Time window width from peak to the right
delta_tr = 4e-2/c0;  % Roll-off time of the window (left and right)
% Pass and stop-band deviation
dev_pass_dB = 0.1;
dev_stop_dB = -50;

% Spreading type (default = spherical)
spread_type = 1;
% Distance from tx to sample (for spreading models)
D = 1e10;

% Debug plots
do_plot = 0;

% Exact eps
eps_r_exact = [];

% LPC extension of data
ext_data = [];


%% Get parameters

if isfield(ops, 'model')
  model = ops.model;
end
if isfield(ops, 'N_band')
  N_band = ops.N_band;
end
if isfield(ops, 'lin_range')
  lin_range = ops.lin_range;
end
if isfield(ops, 'cole_range')
  cole_range = ops.cole_range;
end
if isfield(ops, 'filt_type')
  filt_type = ops.filt_type;
end
if isfield(ops, 'delta_w1')
  delta_w1 = ops.delta_w1;
end
if isfield(ops, 'delta_w2')
  delta_w2 = ops.delta_w2;
end
if isfield(ops, 'delta_tr')
  delta_tr = ops.delta_tr;
end
if isfield(ops, 'dev_pass_dB')
  dev_pass_dB = ops.dev_pass_dB;
end
if isfield(ops, 'dev_stop_dB')
  dev_stop_dB = ops.dev_stop_dB;
end
if isfield(ops, 'spread_type')
  spread_type = ops.spread_type;
end
if isfield(ops, 'D')
  D = ops.D;
end
if isfield(ops, 'N_iter'),
  N_iter = ops.N_iter;
end
if isfield(ops, 'do_plot'),
  do_plot = ops.do_plot;
end
if isfield(ops, 'eps_r_exact'),
  eps_r_exact = ops.eps_r_exact;
end
if isfield(ops, 'ext_data'),
  ext_data = ops.ext_data;
end

% If there are repeated measurment snapshots, average them
if size(S_air, 2) > 1,
  S_air = mean(S_air, 2);
end

if size(S_mat, 2) > 1,
  S_mat = mean(S_mat, 2);
end

if ~isempty(ext_data),
  % Get extension parameters.  Maybe I should make these
  % frequency extents rather than # of samples.
  N_ext_left = ext_data(1);
  N_ext_right = ext_data(2);
  N_pred = ext_data(3);
  
  % Try to extend the data with linear prediction
  [f_new S_air trim_idx] = extend_lpc(f, S_air, N_ext_left, N_ext_right, N_pred);
  [f_new S_mat] = extend_lpc(f, S_mat, N_ext_left, N_ext_right, N_pred);
  f = f_new;
  
end
  
Nf = length(f);
df = f(2)-f(1);
fs = 1/df;

w = blackman(Nf);
zz = zeros(Nf, 1);

%% Do complex baseband representation of frequency-domain data

% Center spectra and transform to time domain
Y = ifft(fftshift(w.*S_air));
Y2 = ifft(fftshift(w.*S_mat));

% Get frequency time and distance ranges
t = [0:length(S_air)-1]/length(S_air)/df;
ff = [0:length(S_air)-1]*df;
rng = t*c0;

% Find the peaks of time-domain spectra.  This will be where
% the kaiser window "center" is located.

% Center of air signal
[val idx] = max(abs(Y));
t0 = t(idx);

% Center of material signal
[val idx] = max(abs(Y2));
tm = t(idx);

% Compute the windows according to input parameters
if strcmp(filt_type, 'kaiser'),
  fsamp = 1/df;
  fcuts = [-(delta_tr+delta_w1) -delta_w1 delta_w2 (delta_w2+delta_tr)]+t0;
  mags = [0 1 0];
  devs = [10^(dev_stop_dB/20) (10^(dev_pass_dB/20)-1) 10^(dev_stop_dB/20)];
  [n Wn beta ftype] = kaiserord(fcuts, mags, devs, fsamp);
  if (Wn(2) > 1),
    error(sprintf('kaiser window with band stop at %g.  Need denser frequency sampling.', Wn(2)));
  end
  n = n + rem(n,2);
  hh = fir1(n, Wn, ftype, kaiser(n+1,beta),'noscale');
  N = 2^ceil(log2(2*n));
  [H f1] = freqz(hh,1,N,fsamp);

  sa2 = conv(S_air, hh, 'same');
  sm2 = conv(S_mat, hh, 'same');

elseif strcmp(filt_type, 'kaiser2'),  

  % Get window for air
  fsamp = 1/df;
  fcuts = [-(delta_tr+delta_w1) -delta_w1 delta_w2 (delta_w2+delta_tr)]+t0;
  mags = [0 1 0];
  devs = [10^(dev_stop_dB/20) (10^(dev_pass_dB/20)-1) 10^(dev_stop_dB/20)];
  [n Wn beta ftype] = kaiserord(fcuts, mags, devs, fsamp);
  if (Wn(2) > 1),
    error(sprintf('kaiser window with band stop at %g.  Need denser frequency sampling.', Wn(2)));
  end
  n = n + rem(n,2);
  hh = fir1(n, Wn, ftype, kaiser(n+1,beta),'noscale');
  N = 2^ceil(log2(2*n));
  [H f1] = freqz(hh,1,N,fsamp);

  % Get window for sample
  fcuts = [-(delta_tr+delta_w1) -delta_w1 delta_w2 (delta_w2+delta_tr)]+tm;
  [n_m Wn_m beta_m ftype_m] = kaiserord(fcuts, mags, devs, fsamp);
  if (Wn(2) > 1),
    error(sprintf('kaiser window with band stop at %g.  Need denser frequency sampling.', Wn(2)));
  end
  
  n_m = n_m + rem(n_m,2);
  hh_m = fir1(n_m, Wn_m, ftype_m, kaiser(n_m+1,beta_m),'noscale');
  N_m = 2^ceil(log2(2*n_m));
  [H_m f1_m] = freqz(hh_m,1,N_m,fsamp);

  % Do filtering
  sa2 = conv(S_air, hh, 'same');
  sm2 = conv(S_mat, hh_m, 'same');
  
elseif strcmp(filt_type, 'ellip'),
  % Elliptical model doesn't seem to work very well.  Not recommended.
  N = ellip_order;
  fsamp = 1/df;
  wl = 2*(t0-delta_w1)/fsamp;
  wh = 2*(t0+delta_w2)/fsamp;
  [B A] = ellip(N, dev_pass_dB, -dev_stop_dB, [wl wh]);
  [H f1] = freqz(B,A,length(S_air),fsamp);
  
  % Do filtering
  dN = round(N/2);
  sa2 = filter(B, A, S_air); %sa2 = sa2(dN + [0:length(S_air)-1]);
  sm2 = filter(B, A, S_mat); %sm2 = sm2(dN + [0:length(S_mat)-1]);
elseif strcmp(filt_type, 'none'),
  sa2 = S_air(:);
  sm2 = S_mat(:);
end

if (do_plot),
  if strcmp(filt_type, 'kaiser') || strcmp(filt_type, 'kaiser2'),
  figure(1);
  Yk = ifft(fftshift(w.*sa2));
  Yk2 = ifft(fftshift(w.*sm2));
  t2 = t*1e9;
  plot(t2, 20*log10(abs(Y)/max(abs(Y))), 'b', ...
       t2, 20*log10(abs(Yk)/max(abs(Yk))), 'b--', ...
       t2, 20*log10(abs(Y2)/max(abs(Y2))), 'r', ...
       t2, 20*log10(abs(Yk2)/max(abs(Yk2))), 'r--', ...
       f1*1e9, 20*log10(abs(H)), 'g');
  legend('Measured h(t) (air)', ' " Windowed', 'Measured h(t) (mat)', ' " Windowed', 'Kaiser Window');
  xlim((t0+[-2*(delta_w1+delta_tr) 2*(delta_w2+delta_tr)])*1e9);
  ylim([-80 0]);
  xlabel('Time (ns)');
  end
end

if ~isempty(ext_data),
  % Trim extended data off after time gate filter.
  sa2 = sa2(trim_idx);
  sm2 = sm2(trim_idx);
  f = f(trim_idx);

  Nf = length(f);
  df = f(2)-f(1);
  fs = 1/df;
end

%% Calibrate the material measurement using the air-measurement.  This
%% basically removes the effect of the antennas and free-space propagation
%% up to the material boundaries.

k0 = 2*pi*f(:)/c0;

if (spread_type == 0),
  sm3 = sm2./(sa2.*exp(j*k0*d));         % No spherical wave spreading (1D)
elseif (spread_type == 1),
  sm3 = sm2./(sa2.*exp(j*k0*d)*(D+d)/D);  % Spherical wave spreading (3D)
elseif (spread_type == 2),
  sm3 = sm2./(sa2.*exp(j*k0*d)*sqrt((D+d)/D));  % Spherical wave spreading (2D)
else
  error('Invalid spread type.');
end

%% Use the GPU-accelerated fitting code based on the model type

%t_gpu1 = time;
if strcmp(model, 'lin'),

  % Linear iterative fitting
  [f_out, eps_r, s21_fit] = fit_eps_lin_c(f, sm3(:).', d, D, lin_range.eps_real, lin_range.eps_imag, lin_range.band, N_iter, spread_type);

elseif strcmp(model, 'cole');

  % Cole-Cole model
  [cole_parm, eps_r, s21_fit] = fit_eps_cole_c(f, sm3(:).', d, D, cole_range, 1);
  f_out = f;
else
  error('Invalid model specified.');
end
%t_gpu2 = time;

%% If exact values for eps_r were provided, compute model for that case.
if ~isempty(eps_r_exact),
  %tmp = get_S_tline2(d, D, eps_r_exact, f, 10, spread_type);
  tmp = get_S_tline3(d, D, eps_r_exact, f, 10, spread_type);
  S21_exact = tmp.S21;
end

%% Plots and debug information

if (do_plot),

  figure(2);
  legends = {'Measured', 'Modeled Fit'};
  if (do_plot == 1),
    plot(f/1e9, abs(sm3), 'b', f/1e9, abs(s21_fit), 'r');
    if ~isempty(eps_r_exact),
      hold on;
      plot(f/1e9, abs(S21_exact), 'k');
      hold off;
      legends{end+1} = 'Exact';
    end
    title('Windowed |S(f)|');
  elseif (do_plot == 2),
    plot(f/1e9, real(sm3), 'b', f/1e9, real(s21_fit), 'r');
    if ~isempty(eps_r_exact),
      hold on;
      plot(f/1e9, real(S21_exact), 'k');
      hold off;
      legends{end+1} = 'Exact';
    end
    title('Windowed Real {S(f)}');
  end
  xlabel('GHz');
  legend(legends);

  figure(3);
  legends = {'Measured', 'Modeled Fit'};
  if (do_plot == 1),
    plot(f/1e9, angle(sm3), 'b', f/1e9, angle(s21_fit), 'r');
    if ~isempty(eps_r_exact),
      hold on;
      plot(f/1e9, angle(S21_exact), 'k');
      hold off;
      legends{end+1} = 'Exact';
    end
    title('Windowed <S(f)');
  elseif (do_plot == 2),
    plot(f/1e9, imag(sm3), 'b', f/1e9, imag(s21_fit), 'r');
    if ~isempty(eps_r_exact),
      hold on;
      plot(f/1e9, imag(S21_exact), 'k');
      hold off;
      legends{end+1} = 'Exact';
    end
    title('Windowed Imag {S(f)}');
  end
  xlabel('GHz');
  legend(legends);

  figure(4);
  title('Complex Permittivity');
  plot(f_out/1e9, real(eps_r), '-bx', f_out/1e9, imag(eps_r), '-r*');
  legend('eps_r', 'eps_i', 'location', 'west');
  xlabel('GHz');
end

% Print fitting error
err = sqrt(mean(abs(s21_fit(:) - sm3(:)).^2));
err_den = sqrt(mean(abs(sm3(:)).^2));
fprintf('Mean error: %g %%\n', err/err_den*100);

d_fit = [];
d_fit.err = err/err_den;

%% Print required GPU time
%fprintf('GPU Time = %.2f seconds.\n', t_gpu2 - t_gpu1);

% Return values
f_fit = f_out;
eps_r_fit = eps_r;

