function [f_out, eps_r, s21_fit] = fit_eps_lin_c(f_in, s21_meas, d, D, eps_real_rng, eps_imag_rng, band_rng, N_iter, spread_type);

% [f_out, eps_r, s21_fit] = fit_eps_lin_c(f, s21_meas, d, D, eps_real_rng, eps_imag_rng, band_rng, N_iter, spread_type);
%
% Fits a measured response to modeled one using the linear fitter.  This
% optimization starts by fitting the whole measurement to a constant eps_r.
% Then, an iterative procedure is used to update band points.  Produces
% very good fits.
%
% Inputs:
%       f               Frequency range of data
%       s21_meas        Measured and calibrated s21 response
%       d               Thickness of slab
%       D               distance from source to slab
%                       If a single number uses old formulation
%                       If a vector assumes it is [D D']
%       eps_real_rng    Sample points for real part of eps_r [start stop N log?]
%       eps_imag_rng    Sample points for imag part of eps_r [start stop N log?]
%       band_rng        Sample points for bands [start stop N log?]
%       N_iter          Number of iterations to perform

if (nargin < 9),
  spread_type = 1;
end

use_gpu = 1;
N_refl_order = 10;
log_level = 3;

fname_parm = 'fit_eps.in';
fname_parm_s = 'fit_eps_s.in';

% Generate parameter input file
f = fopen(fname_parm, 'wb');
fprintf(f, 'gpu %d\n', use_gpu);
if length(D) == 1,
  fprintf(f, 'D %e\n', D);
else
  fprintf(f, 'D %e\n', D(1));
  fprintf(f, 'Dp %e\n', D(2));
end
fprintf(f, 'd %d\n', d);
fprintf(f, 'eps_real %e %e %d %d\n', eps_real_rng);
fprintf(f, 'eps_imag %e %e %d %d\n', eps_imag_rng);
fprintf(f, 'band %e %e %d %d\n', band_rng);
fprintf(f, 'f %e %e %d\n', f_in(1), f_in(end), length(f_in));
fprintf(f, 'iter %d\n', N_iter);

fprintf(f, 'ord %d\n', N_refl_order);
fprintf(f, 'log %d\n', log_level);
fprintf(f, 'spread %d\n', spread_type);
fclose(f);

% Store S-parameters
f = fopen(fname_parm_s, 'wb');
for ii=1:length(s21_meas),
  fprintf(f, '%e %e\n', real(s21_meas(ii)), imag(s21_meas(ii)));
end
fclose(f);

% Execute the optimizer
fname_out = 'eps_r.out';
unix(sprintf('rm -rf %s', fname_out));
unix(sprintf('./fit_eps_lin %s %s', fname_parm, fname_parm_s));

% Read results
f = fopen(fname_out, 'rb');
data = fscanf(f, '%e', Inf);
fclose(f);
data = reshape(data, 2, length(data)/2);
eps_r = data(1, :) + j*data(2,:);

if (band_rng(4)),
  f_out = exp([0:band_rng(3)-1]/(band_rng(3)-1)*(log(band_rng(2)) - log(band_rng(1))) + log(band_rng(1)));
else
  f_out = [0:band_rng(3)-1]/(band_rng(3)-1)*(band_rng(2) - band_rng(1)) + band_rng(1);
end

% Interpolate values to get eps profile
eps_r_all = interp1(f_out, eps_r, f_in);

% Compute fitted response for all bands
if (length(D) == 1),
  tmp = get_S_tline2(d, D, eps_r_all, f_in, N_refl_order, spread_type);
else
  tmp = get_S_tline3(d, D(1), D(2), eps_r_all, f_in, N_refl_order, spread_type);
end
s21_fit = tmp.S21;

